import { EnchantmentType, ItemLockMode, ItemStack, StructureSaveMode, world } from "@minecraft/server";
import Definitions from "../configuration";
import { LightLevel, parseID, Vector } from "./utils";
export class Belt {
    id;
    slot;
    constructor(data) {
        let ID = typeof data === 'number' ? data : data.getDynamicProperty('ID');
        if (typeof data !== 'number') {
            this.slot = data;
            if (!ID)
                ID = Date.now();
            data.setDynamicProperty('ID', ID);
        }
        this.id = ID;
    }
    get data() {
        const data = world.getDynamicProperty(`rts:${this.id}_inventory`);
        return data ? JSON.parse(data) : [];
    }
    set data(data) {
        world.setDynamicProperty(`rts:${this.id}_inventory`, JSON.stringify(data));
    }
    loadInventory(entity) {
        entity.setDynamicProperty('ID', this.id);
        const data = world.structureManager.get(`rts:${this.id}`);
        if (!data)
            return;
        const { dimension, location } = entity;
        location.y = dimension.heightRange.min + 1;
        const original = world.structureManager.createFromWorld(`rts:tmp_${this.id}`, dimension, location, location, {
            saveMode: StructureSaveMode.Memory,
            includeEntities: false,
        });
        world.structureManager.place(data, dimension, location);
        const barrel = dimension.getBlock(location);
        const container = barrel?.inventory.container;
        const inv = entity.inventory?.container;
        for (let i = 0; i < inv.size; i++) {
            const item = container.getItem(i);
            inv.setItem(i, item);
        }
        container.clearAll();
        world.structureManager.place(original, dimension, location);
        world.structureManager.delete(original);
        world.structureManager.delete(data);
    }
    saveInventory(entity, slot) {
        const { dimension, location } = entity;
        location.y = dimension.heightRange.min + 1;
        const ID = slot?.getDynamicProperty('ID') ?? this.id;
        try {
            world.structureManager.delete(`rts:tmp_${ID}`);
            world.structureManager.delete(`rts:${ID}`);
        }
        catch { }
        const original = world.structureManager.createFromWorld(`rts:tmp_${ID}`, dimension, location, location, {
            saveMode: StructureSaveMode.Memory,
            includeEntities: false,
        });
        const barrel = dimension.getBlock(location);
        barrel.setType('barrel');
        const container = barrel?.inventory.container;
        const inv = entity.inventory?.container;
        const items = [];
        const lore = [];
        for (let i = 0; i < inv.size; i++) {
            const item = inv.getItem(i);
            if (!item) {
                lore.push(`§r- §8§o(...Empty)`);
                continue;
            }
            container.setItem(i, item);
            const light = LightLevel[item.typeId];
            // @ts-ignore
            items.push({ slot: i, ...serializeItem(item) });
            lore.push(`§r- ${parseID(item.typeId)}` + (item.amount > 1 ? ` x${item.amount}` : '') + (light ? ' []' : ''));
        }
        this.data = items;
        if (!slot)
            slot = this.slot;
        if (slot?.hasItem() && slot.typeId === 'rts:tool_belt')
            slot?.setLore([
                '§r§lContents:',
                ...lore
            ]);
        world.structureManager.createFromWorld(`rts:${ID}`, dimension, location, location, {
            saveMode: StructureSaveMode.World,
            includeEntities: false,
        });
        container.clearAll();
        world.structureManager.place(original, dimension, location);
        world.structureManager.delete(original);
    }
}
export function serializeItem(item) {
    return {
        typeId: item.typeId,
        amount: item.amount,
        lore: item.getLore(),
        nameTag: item.nameTag,
        lockMode: item.lockMode,
        keepOnDeath: item.keepOnDeath,
        canDestroy: item.getCanDestroy(),
        canPlaceOn: item.getCanDestroy(),
        damage: item.durability?.damage,
        enchantments: item.enchantable?.getEnchantments(),
        dynamicProperties: item.getDynamicPropertyIds().map(id => ([id, item.getDynamicProperty(id)])),
    };
}
export function parseItem(data) {
    try {
        const { typeId, amount, nameTag, lore, canDestroy, canPlaceOn, damage, enchantments, dynamicProperties } = data;
        const item = new ItemStack(typeId, amount);
        item.nameTag = nameTag;
        item.setLore(lore);
        item.setCanDestroy(canDestroy);
        item.setCanPlaceOn(canPlaceOn);
        if (damage)
            item.durability.damage = damage;
        enchantments?.forEach(({ type, level }) => {
            const ench = new EnchantmentType(type.id);
            item.enchantable?.addEnchantment({ type: ench, level });
        });
        dynamicProperties.forEach(([id, value]) => item.setDynamicProperty(id, value));
        return item;
    }
    catch { }
}
export function unlockItems(player, type) {
    const inv = player.inventory.container;
    for (let i = 0; i < inv.size; i++) {
        const item = inv.getItem(i);
        if (item?.typeId !== type || item.lockMode === ItemLockMode.none)
            continue;
        item.lockMode = ItemLockMode.none;
        inv.setItem(i, item);
    }
}
export function getTooBelts(player) {
    const inv = player.inventory.container;
    const belts = [];
    for (let i = 0; i < inv.size; i++) {
        const slot = inv.getSlot(i);
        if (!slot.hasItem() || slot.typeId !== 'rts:tool_belt')
            continue;
        const ID = slot.getDynamicProperty('ID');
        if (ID)
            belts.push(new Belt(slot));
    }
    return belts;
}
export function getToolDefinition(item) {
    for (const definition of Definitions) {
        const { tool } = definition;
        if (item.typeId.match(tool.regex))
            return definition;
        const iTags = item.getTags();
        if (!iTags.length)
            continue;
        for (const tag of tool.tags) {
            if (!iTags.some(t => t.match(tag)))
                continue;
            return definition;
        }
    }
}
export function validateDefinition(tool, permutation, interaction) {
    const { blocks, interactions } = tool;
    const data = (interaction ? interactions : blocks);
    if (data?.types.some(t => permutation.type.id.match(t)))
        return true;
    const bTags = permutation.getTags();
    return !!(bTags.length && data?.tags.some(t => bTags.some(bT => bT.match(t))));
}
export function setupContainer(player, belt, name, offset = {}) {
    const entity = player.dimension.spawnEntity('rts:belt_container', Vector.add(player.getHeadLocation(), offset));
    entity.nameTag = name ?? 'Tool Belt';
    entity.setDynamicProperty('ID', belt.id);
    entity.addTag(`rts:${player.id}`);
    const tameable = entity.getComponent('tameable');
    tameable.tame(player);
    belt.loadInventory(entity);
    return entity;
}
export function lockBelts(player) {
    const inv = player.inventory.container;
    for (let i = 0; i < inv.size; i++) {
        const slot = inv.getSlot(i);
        if (!slot.hasItem() || !slot.typeId.startsWith('rts:tool_belt'))
            continue;
        slot.lockMode = i === player.selectedSlotIndex ? ItemLockMode.slot : ItemLockMode.inventory;
    }
}
export function unlockBelts(player) {
    const inv = player.inventory.container;
    for (let i = 0; i < inv.size; i++) {
        const slot = inv.getSlot(i);
        if (!slot.hasItem() || !slot.typeId.startsWith('rts:tool_belt'))
            continue;
        slot.lockMode = ItemLockMode.none;
    }
}
